@@ -196,7 +196,6 @@ class MeshData:
     def getVerticesAsByteArray(self) -> Optional[bytes]:
         if self._vertices is None:
             return None
-        # FIXME cache result
         return self._vertices.tostring()
 
     ##  Get all normals of this mesh as a bytearray
@@ -205,7 +204,6 @@ class MeshData:
     def getNormalsAsByteArray(self) -> Optional[bytes]:
         if self._normals is None:
             return None
-        # FIXME cache result
         return self._normals.tostring()
 
     ##  Get all indices as a bytearray
@@ -214,19 +212,16 @@ class MeshData:
     def getIndicesAsByteArray(self) -> Optional[bytes]:
         if self._indices is None:
             return None
-        # FIXME cache result
         return self._indices.tostring()
 
     def getColorsAsByteArray(self) -> Optional[bytes]:
         if self._colors is None:
             return None
-        # FIXME cache result
         return self._colors.tostring()
 
     def getUVCoordinatesAsByteArray(self) -> Optional[bytes]:
         if self._uvs is None:
             return None
-        # FIXME cache result
         return self._uvs.tostring()
 
     #######################################################################
@@ -259,7 +254,7 @@ class MeshData:
     ##  Gets transformed convex hull points
     #
     #   \return \type{numpy.ndarray} the vertices which describe the convex hull
-    def getConvexHullTransformedVertices(self, transformation: Matrix) -> numpy.ndarray:
+    def getConvexHullTransformedVertices(self, transformation: Matrix) -> Optional[numpy.ndarray]:
         vertices = self.getConvexHullVertices()
         if vertices is not None:
             return transformVertices(vertices, transformation)
@@ -360,10 +355,11 @@ def approximateConvexHull(vertex_data: numpy.ndarray, target_count: int) -> Opti
     start_time = time()
 
     input_max = target_count * 50   # Maximum number of vertices we want to feed to the convex hull algorithm.
-    unit_size = 0.125               # Initial rounding interval. i.e. round to 0.125.
+    unit_size = 0.0125             # Initial rounding interval. i.e. round to 0.125.
+    max_unit_size = 0.01
 
     # Round off vertices and extract the uniques until the number of vertices is below the input_max.
-    while len(vertex_data) > input_max:
+    while len(vertex_data) > input_max and unit_size <= max_unit_size:
         new_vertex_data = uniqueVertices(roundVertexArray(vertex_data, unit_size))
         # Check if there is variance in Z value, we need it for the convex hull calculation
         if numpy.amin(new_vertex_data[:, 1]) != numpy.amax(new_vertex_data[:, 1]):
@@ -381,7 +377,7 @@ def approximateConvexHull(vertex_data: numpy.ndarray, target_count: int) -> Opti
     hull_result = scipy.spatial.ConvexHull(vertex_data)
     vertex_data = numpy.take(hull_result.points, hull_result.vertices, axis=0)
 
-    while len(vertex_data) > target_count:
+    while len(vertex_data) > target_count and unit_size <= max_unit_size:
         vertex_data = uniqueVertices(roundVertexArray(vertex_data, unit_size))
         hull_result = scipy.spatial.ConvexHull(vertex_data)
         vertex_data = numpy.take(hull_result.points, hull_result.vertices, axis=0)
